<?php

namespace hitb\facades\user;

use hitb\core\security\AuthenticationService;
use hitb\core\security\InvalidCredentialsException;
use hitb\core\session\SessionService;
use hitb\util\converter\Converter;

use hitb\util\converter\Converters;
use Phalcon\Session\AdapterInterface;

/**
 * Default implementation of {@link UserFacade}.
 */
class DefaultUserFacade implements UserFacade {

	const SID = "user";

	private $authenticationService;
	private $sessionService;
	private $userService;
	private $userConverter;
	private $reverseUserConverter;
	private $security;

	/**
	 * Retrieves the currently logged in user, if any.
	 *
	 * @return the current user data, or null if not logged in
	 */
	public function getCurrentUser() {

		if (!$this->getSessionService()->has(self::SID)) {
			return null;
		}

		return $this->getSessionService()->get(self::SID);
	}

	/**
	 * Checks the given credentials and creates a session for the user.
	 * @param $uid
	 * @param $pwd
	 * @return mixed the user data
	 * @throws InvalidCredentialsException if an error occurs
	 */
	public function login($uid, $pwd) {

		$user = $this->getAuthenticationService()->verifyCredentials($uid, $pwd);

		$userData = $this->getUserConverter()->convert($user);

		$this->getSessionService()->set(self::SID, $userData);

		return $userData;
	}

	/**
	 * Refreshes the currently logged in user.
	 */
	public function refreshCurrentUser() {
		$loggedon = $this->getSessionService()->get(self::SID);

		$user = $this->getUserService()->get($loggedon->getUid());

		$userData = $this->getUserConverter()->convert($user);

		$this->getSessionService()->set(self::SID, $userData);
	}

	/**
	 * Retrieves a list of employees.
	 *
	 * @return mixed
	 */
	public function getEmployees()
	{
		$users = $this->getUserService()->getStandardUsers();

		return Converters::convertAll($users, $this->getUserConverter());
	}

	/**
	 * Removes the session for the current user.
	 *
	 */
	public function logout() {
		$this->getSessionService()->remove(self::SID);
	}

	/**
	 * Creates a new guest user.
	 *
	 * @param $guest
	 * @return mixed
	 * @throws \Exception
	 */
	public function createGuestUser($guest, $pwd)
	{
		$user = $this->getCurrentUser();
		if ($user->getLevel() < 4)
			throw new \Exception("Only moderators or admins can create guest users");

		$model = $this->getReverseUserConverter()->convert($guest);

		// Default values for guests
		$model->setPwd($this->security->hash($pwd));
		$model->setGuest(1);
		$model->setLevel(1);

		$result = $this->getUserService()->save($model);

		return $this->getUserConverter()->convert($result);
	}

	/**
	 * Updates the profile of the guest user.
	 *
	 * @param $guest
	 * @return mixed
	 * @throws \Exception
	 */
	public function updateGuestUser($guest)
	{
		$user = $this->getCurrentUser();
		if ($user->getGuest() != 1)
			throw new \Exception("Only guest users can update their profile");

		$model = $this->getReverseUserConverter()->convertExisting($guest, $this->getUserService()->get($user->getUid()));

		// Only the logged on guest may update his own profile
		$model->setUid($user->getUid());

		$result = $this->getUserService()->save($model);

		// Make sure the updated data is reflected in the cache
		$this->refreshCurrentUser();
		
		return $this->getUserConverter()->convert($result);
	}

	/**
	 * @return mixed
	 */
	public function getAuthenticationService()
	{
		return $this->authenticationService;
	}

	/**
	 * @param mixed $authenticationService
	 */
	public function setAuthenticationService($authenticationService)
	{
		$this->authenticationService = $authenticationService;
	}

	/**
	 * @return mixed
	 */
	public function getSessionService()
	{
		return $this->sessionService;
	}

	/**
	 * @param mixed $sessionService
	 */
	public function setSessionService($sessionService)
	{
		$this->sessionService = $sessionService;
	}

	/**
	 * @return mixed
	 */
	public function getUserService()
	{
		return $this->userService;
	}

	/**
	 * @param mixed $userService
	 */
	public function setUserService($userService)
	{
		$this->userService = $userService;
	}

	/**
	 * @return mixed
	 */
	public function getUserConverter()
	{
		return $this->userConverter;
	}

	/**
	 * @param mixed $userConverter
	 */
	public function setUserConverter($userConverter)
	{
		$this->userConverter = $userConverter;
	}

	/**
	 * @return mixed
	 */
	public function getReverseUserConverter()
	{
		return $this->reverseUserConverter;
	}

	/**
	 * @param mixed $reverseUserConverter
	 */
	public function setReverseUserConverter($reverseUserConverter)
	{
		$this->reverseUserConverter = $reverseUserConverter;
	}

	/**
	 * @return mixed
	 */
	public function getSecurity()
	{
		return $this->security;
	}

	/**
	 * @param mixed $security
	 */
	public function setSecurity($security)
	{
		$this->security = $security;
	}

}

