<?php

namespace hitb\storefront\controllers;

use hitb\core\model\Users;
use hitb\core\security\InvalidCredentialsException;

use hitb\facades\user\data\UserData;
use hitb\storefront\forms\LoginForm;
use hitb\storefront\forms\UserForm;

/**
 * Provides user operations.
 */
class UserController extends AbstractController
{
	/**
	 * Shows a list of employees.
	 */
    public function listAction()
    {
		$users = $this->getUserFacade()->getEmployees();

		$this->view->users = $users;
    }

	/**
	 * Handles the add guest user functionality.
	 *
	 * @return mixed
	 */
	public function addAction() {
		$form = new UserForm();

		if ($this->request->isPost()) {

			$form->populate($this->request);

			if ($form->isValid()) {
				return $this->addGuestUser($form);
			}
		}

		return $this->showAddForm($form);
	}

	/**
	 * Shows the add guest user form.
	 *
	 * @param $form
	 */
	protected function showAddForm($form) {
        $this->view->form = $form;
	}

	/**
	 * Creates the guest user and handles any failures.
	 *
	 * @param $form
	 * @internal param $uid
	 * @internal param $pwd
	 */
	protected function addGuestUser($form) {
		try {
			$userData = $this->createData($form);

			// Add the guest user
			$pwd = uniqid();
			$guest = $this->getUserFacade()->createGuestUser($userData, $pwd);

			$this->view->guest = $guest;
			$this->view->pwd = $pwd;

			$this->view->pick("user/added");
		} catch (\Exception $exception) {
			$form->addValidationConstraint('uid', 'invalid', $exception->getMessage());
			return $this->showAddForm($form);
		}
	}

	/**
	 * Handles the user add functionality.
	 *
	 * @return mixed
	 */
	public function updateAction() {

		$form = new UserForm();

		if ($this->request->isPost()) {

			$form->populate($this->request);

			if ($form->isValid()) {
				return $this->updateGuestUser($form);
			}
		} else {
			$user = $this->getUserFacade()->getCurrentUser();
			$form->setRestaurant($user->getRestaurant());
		}

		return $this->showUpdateForm($form);
	}

	/**
	 * Shows the update guest user profile form.
	 *
	 * @param $form
	 */
	protected function showUpdateForm($form) {
		$this->view->form = $form;
	}

	/**
	 * Updates the guest user profile and handles any failures.
	 *
	 * @param $form
	 * @internal param $uid
	 * @internal param $pwd
	 */
	protected function updateGuestUser($form) {
		try {
			$userData = $this->createData($form);

			// Update profile
			$guest = $this->getUserFacade()->updateGuestUser($userData);

			$this->view->guest = $guest;
			$this->view->pick("user/updated");
		} catch (\Exception $exception) {
			$form->addValidationConstraint('uid', 'invalid', $exception->getMessage());
			return $this->showAddForm($form);
		}
	}

	/**
	 * Allows moderators to reset other employees' accounts.
	 */
	public function resetAction() {
		// functionality disabled for now
	}

	/**
	 * Creates a new {@link UserData} instance from the given {@link UserForm}.
	 *
	 * @param $form
	 * @return UserData
	 */
	protected function createData(UserForm $form) {
		$data = new UserData();
		$data->setUid($form->getUid());
		$data->setLevel($form->getLevel());
		$data->setReset($form->getReset());
		$data->setRestaurant($form->getRestaurant());

		return $data;
	}

	/**
	 * @return mixed
	 */
	protected function getUserFacade() {
		return $this->di->get('userFacade');
	}

	/**
	 * @return mixed
	 */
	protected function getReverseUserConverter() {
		return $this->di->get('reverseUserConverter');
	}

}

